var path_1 = require('path');
var commander_1 = require('commander');
var repl_1 = require('repl');
var util_1 = require('util');
var fs_1 = require('fs');
var Module = require('module');
var extend = require('xtend');
var vm_1 = require('vm');
var typescript_node_1 = require('../typescript-node');
var program = new commander_1.Command('ts-node');
var pkg = require('../../package.json');
program.option('-e, --eval [code]', 'Evaluate code');
program.option('-p, --print [code]', 'Evaluate code and print result');
program.option('--compiler [name]', 'Specify a custom TypeScript compiler');
program.option('--ignoreWarnings [codes]', 'Specify a custom TypeScript compiler', list);
program.option('--configFile [path]', 'Specify a custom TypeScript compiler');
program.version(pkg.version);
program.usage('[options] [ -e script | script.js ] [arguments]');
program.parse(process.argv);
var EVAL_FILENAME = '[eval].ts';
var cwd = process.cwd();
var opts = program.opts();
var print = opts.print;
var eval = opts.eval;
var code = eval == null ? print : eval;
var compiler = typescript_node_1.register(opts);
var files;
var service;
if (typeof code === 'string') {
    createService();
    global.__filename = EVAL_FILENAME;
    global.__dirname = cwd;
    var module_1 = new Module(global.__filename);
    module_1.filename = global.__filename;
    module_1.paths = Module._nodeModulePaths(global.__dirname);
    global.exports = module_1.exports;
    global.module = module_1;
    global.require = module_1.require.bind(module_1);
    var result = _eval(code, global.__filename);
    if (print != null) {
        var output = typeof result === 'string' ? result : util_1.inspect(result);
        process.stdout.write(output + '\n');
    }
}
else {
    if (program.args.length) {
        var index = 2;
        var skip = false;
        for (; index < process.argv.length; index++) {
            if (skip) {
                skip = false;
                continue;
            }
            var arg = process.argv[index];
            if (arg.charAt(0) === '-') {
                var value = opts[arg.substr(2)];
                if (value && value !== true) {
                    skip = true;
                }
                continue;
            }
            break;
        }
        var args = process.argv.slice(index);
        args[0] = path_1.resolve(args[0]);
        process.argv = ['node'].concat(args);
        process.execArgv.unshift(__filename);
        Module.runMain();
    }
    else {
        createService();
        startRepl();
    }
}
function _eval(text, filename) {
    if (!files[filename]) {
        files[filename] = { version: 0, text: '' };
    }
    var file = files[filename];
    file.text = text;
    file.version++;
    var output = service.getEmitOutput(filename);
    var diagnostics = typescript_node_1.getDiagnostics(service, filename, compiler.options);
    if (diagnostics.length) {
        throw typescript_node_1.createError(diagnostics, compiler.ts);
    }
    var result = output.outputFiles[1].text;
    var sourceMap = output.outputFiles[0].text;
    var code = result.replace('//# sourceMappingURL=' + filename.replace(/\.ts$/, '.js.map'), '//# sourceMappingURL=' + typescript_node_1.getInlineSourceMap(sourceMap, filename, text));
    return vm_1.runInThisContext(result, filename);
}
function createService() {
    var ts = compiler.ts, registry = compiler.registry, config = compiler.config;
    files = {};
    service = ts.createLanguageService({
        getScriptFileNames: function () {
            return config.fileNames.concat(Object.keys(files));
        },
        getScriptVersion: function (fileName) { return files[fileName] && files[fileName].version.toString(); },
        getScriptSnapshot: function (fileName) {
            var file = files[fileName];
            if (file) {
                return ts.ScriptSnapshot.fromString(file.text);
            }
            try {
                return ts.ScriptSnapshot.fromString(fs_1.readFileSync(fileName, 'utf-8'));
            }
            catch (e) {
                return;
            }
        },
        getCurrentDirectory: function () { return cwd; },
        getScriptIsOpen: function () { return true; },
        getCompilationSettings: function () { return extend(config.options, { sourceMap: true }); },
        getDefaultLibFileName: function (options) { return ts.getDefaultLibFilePath(options); }
    }, registry);
}
function startRepl() {
    return repl_1.start({
        prompt: '> ',
        input: process.stdin,
        output: process.stdout,
        eval: replEval,
        useGlobal: true
    });
}
function replEval(code, context, filename, callback) {
    var err;
    var result;
    try {
        result = _eval(code, EVAL_FILENAME);
    }
    catch (e) {
        err = e;
    }
    callback(err, result);
}
function list(value) {
    return value.split(/ *, */);
}
//# sourceMappingURL=ts-node.js.map