var tsconfig = require('tsconfig');
var path_1 = require('path');
var fs = require('fs');
var os_1 = require('os');
var sourceMapSupport = require('source-map-support');
var extend = require('xtend');
var arrify = require('arrify');
var cwd = process.cwd();
var readFileSync = fs.readFileSync;
exports.EXTENSIONS = ['.ts', '.tsx'];
function readConfig(fileName, ts) {
    var config = fileName ? tsconfig.readFileSync(fileName) : {
        files: [],
        compilerOptions: {}
    };
    config.compilerOptions = extend({
        target: 'es5'
    }, config.compilerOptions, {
        module: 'commonjs'
    });
    return ts.parseConfigFile(config, ts.sys, fileName);
}
function register(opts) {
    var options = extend(opts);
    var maps = {};
    var files = {};
    options.compiler = options.compiler || 'typescript';
    options.ignoreWarnings = arrify(options.ignoreWarnings);
    options.configFile = options.configFile ?
        path_1.resolve(cwd, options.configFile) :
        tsconfig.resolveSync(cwd);
    var ts = require(options.compiler);
    var config = readConfig(options.configFile, ts);
    if (config.errors.length) {
        throw createError(config.errors, ts);
    }
    var serviceHost = {
        getScriptFileNames: function () { return config.fileNames.concat(Object.keys(files)); },
        getScriptVersion: function (fileName) { return 'node'; },
        getScriptSnapshot: function (fileName) {
            try {
                return ts.ScriptSnapshot.fromString(readFileSync(fileName, 'utf-8'));
            }
            catch (e) {
                return;
            }
        },
        getCurrentDirectory: function () { return cwd; },
        getScriptIsOpen: function () { return true; },
        getCompilationSettings: function () { return config.options; },
        getDefaultLibFileName: function (options) { return ts.getDefaultLibFilePath(config.options); }
    };
    var registry = ts.createDocumentRegistry();
    var service = ts.createLanguageService(serviceHost, registry);
    var hasSourceMap = config.options.sourceMap;
    var retrieveSourceMap = sourceMapSupport.retrieveSourceMap;
    sourceMapSupport.install({
        handleUncaughtExceptions: false,
        retrieveSourceMap: function (filename) {
            var map = maps && maps[filename];
            if (map) {
                return { map: map, url: null };
            }
            return retrieveSourceMap(filename);
        }
    });
    function compile(fileName) {
        files[fileName] = true;
        var output = service.getEmitOutput(fileName);
        var result = output.outputFiles[hasSourceMap ? 1 : 0].text;
        if (hasSourceMap) {
            maps[output.outputFiles[0].name] = output.outputFiles[0].text;
        }
        var diagnostics = getDiagnostics(service, fileName, options);
        if (diagnostics.length) {
            throw createError(diagnostics, ts);
        }
        return result;
    }
    function loader(m, fileName) {
        return m._compile(compile(fileName), fileName);
    }
    exports.EXTENSIONS.forEach(function (extension) {
        require.extensions[extension] = loader;
    });
    return {
        registry: registry,
        ts: ts,
        config: config,
        options: options
    };
}
exports.register = register;
function getDiagnostics(service, fileName, options) {
    return service.getCompilerOptionsDiagnostics()
        .concat(service.getSyntacticDiagnostics(fileName))
        .concat(service.getSemanticDiagnostics(fileName))
        .filter(function (diagnostic) {
        return options.ignoreWarnings.indexOf(String(diagnostic.code)) === -1;
    });
}
exports.getDiagnostics = getDiagnostics;
function formatDiagnostic(diagnostic, ts, cwd) {
    if (cwd === void 0) { cwd = '.'; }
    var message = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
    if (diagnostic.file) {
        var _a = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start), line = _a.line, character = _a.character;
        return path_1.relative(cwd, diagnostic.file.fileName) + " (" + (line + 1) + "," + (character + 1) + "): " + message + " (" + diagnostic.code + ")";
    }
    return message + " (" + diagnostic.code + ")";
}
exports.formatDiagnostic = formatDiagnostic;
function createError(diagnostics, ts) {
    var message = ['Unable to compile TypeScript:']
        .concat(diagnostics.map(function (d) { return formatDiagnostic(d, ts); }))
        .join(os_1.EOL);
    var err = new Error(message);
    err.name = 'TypeScriptError';
    return err;
}
exports.createError = createError;
function getInlineSourceMap(map, fileName, code) {
    var sourceMap = JSON.parse(map);
    sourceMap.file = fileName;
    sourceMap.sources = [fileName];
    sourceMap.sourcesContent = [code];
    delete sourceMap.sourceRoot;
    var inlineSourceMap = new Buffer(JSON.stringify(sourceMap)).toString('base64');
    return 'data:application/json;base64,' + inlineSourceMap;
}
exports.getInlineSourceMap = getInlineSourceMap;
//# sourceMappingURL=typescript-node.js.map